"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.completer = exports.wrapField = exports.createCompletionFilter = void 0;
const semver_1 = require("semver");
const mongodb_constants_1 = require("@mongodb-js/mongodb-constants");
const DEFAULT_SERVER_VERSION = '999.999.999';
const ALL_COMPLETIONS = [
    ...mongodb_constants_1.ACCUMULATORS,
    ...mongodb_constants_1.BSON_TYPES,
    ...mongodb_constants_1.BSON_TYPE_ALIASES,
    ...mongodb_constants_1.CONVERSION_OPERATORS,
    ...mongodb_constants_1.EXPRESSION_OPERATORS,
    ...mongodb_constants_1.JSON_SCHEMA,
    ...mongodb_constants_1.QUERY_OPERATORS,
    ...mongodb_constants_1.STAGE_OPERATORS,
];
function matchesMeta(filter, meta) {
    const metaParts = meta.split(':');
    return filter.some((metaFilter) => {
        const filterParts = metaFilter.split(':');
        return (filterParts.length === metaParts.length &&
            filterParts.every((part, index) => {
                return part === '*' || part === metaParts[index];
            }));
    });
}
function createCompletionFilter(prefix, serverVersion, filterMeta) {
    const currentServerVersion = serverVersion.match(/^(?<version>\d+?\.\d+?\.\d+?)/)?.groups?.version ??
        DEFAULT_SERVER_VERSION;
    return ({ value, version: minServerVersion, meta }) => {
        return (value.toLowerCase().startsWith(prefix.toLowerCase()) &&
            (0, semver_1.gte)(currentServerVersion, minServerVersion) &&
            (!filterMeta || matchesMeta(filterMeta, meta)));
    };
}
exports.createCompletionFilter = createCompletionFilter;
function isValidIdentifier(identifier) {
    if (/[.\s"'()[\];={}]/.test(identifier)) {
        return false;
    }
    try {
        new Function(`"use strict";let ${identifier};`);
        return true;
    }
    catch {
        return false;
    }
}
function wrapField(field, force = false) {
    return force || !isValidIdentifier(field)
        ? `"${field.replace(/["\\]/g, '\\$&')}"`
        : field;
}
exports.wrapField = wrapField;
function normalizeField(field) {
    return typeof field === 'string'
        ? { value: field }
        : {
            value: field.name,
            description: field.description,
        };
}
function completer(prefix = '', options = {}, completions = ALL_COMPLETIONS) {
    const { serverVersion = DEFAULT_SERVER_VERSION, fields = [], meta } = options;
    const completionsFilter = createCompletionFilter(prefix, serverVersion, meta);
    const completionsWithFields = [].concat(completions, fields.flatMap((field) => {
        const { value, description } = normalizeField(field);
        return [
            {
                value: value,
                meta: 'field:identifier',
                version: '0.0.0',
                description,
            },
            {
                value: `$${value}`,
                meta: 'field:reference',
                version: '0.0.0',
                description,
            },
        ];
    }));
    return completionsWithFields
        .filter((completion) => {
        return completionsFilter(completion);
    })
        .map((completion) => {
        return {
            value: completion.value,
            meta: completion.meta,
            score: completion.score ?? 1,
            ...(completion.description && { description: completion.description }),
            ...(completion.snippet && { snippet: completion.snippet }),
        };
    });
}
exports.completer = completer;
//# sourceMappingURL=autocompleter.js.map